/*
 * OpenCA Project
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 1999
 *
 * Copyright (c) 2001 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include "general.h"
#include "url.h"

// URL *getParsedUrl ( char *url_s ) {
URL *URL_new ( char *url_s ) {

	URL *ret = NULL;
	char *tmp_s = NULL;
	char *tmp_s2 = NULL;
	char *tmp_s3 = NULL;
	int len;

	if( !url_s ) return NULL;

	ret = (URL *) malloc ( sizeof( URL ));
	if( !ret ) return NULL;

	memset( ret, 0, sizeof(struct url_data));
	ret->addr = NULL;
	ret->pwd = NULL;
	ret->usr = NULL;
	ret->port= 0;

	if( strncmp("ldap://", url_s, 6 ) == 0) {
		ret->proto = URI_PROTO_LDAP;
		ret->port = DEFAULT_LDAP_PORT;

		tmp_s = &url_s[7];

		if( (tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s2 = strchr( tmp_s, ':' );

			if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
				return NULL;
			}
			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->usr = (char *) malloc (len+1);
			memset( ret->usr, 0, len+1);
			strncpy( ret->usr, tmp_s, len);
			ret->usr[len] = '\x0';
			
			tmp_s = tmp_s2+1;
			tmp_s2 = strchr( tmp_s,'@');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->pwd = (char *) malloc (len+1);
			memset( ret->pwd, 0, len+1);
			strncpy( ret->pwd, tmp_s, len);
			ret->pwd[len] = '\x0';

			tmp_s = tmp_s2+1;
		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset(ret->addr, 0, len+1);
			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
		} else {
			ret->addr = (char *) malloc ( BUFF_MAX_SIZE );
			memset( ret->addr, 0, BUFF_MAX_SIZE );

			strncpy( ret->addr, tmp_s, BUFF_MAX_SIZE);
			ret->port = DEFAULT_LDAP_PORT;
		}

	} else if (strncmp( "file://", url_s, 6 ) == 0) {
		ret->port = -1;
		ret->proto = URI_PROTO_FILE;
		ret->addr = (char *) malloc ( BUFF_MAX_SIZE );
		memset( ret->addr, 0, BUFF_MAX_SIZE );

		strncpy( ret->addr, &url_s[7], BUFF_MAX_SIZE);
	} else if( strncmp("http://", url_s, 6 ) == 0) {
		ret->proto = URI_PROTO_HTTP;
		tmp_s = &url_s[7];

		if( (tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s2 = strchr( tmp_s, ':' );

			if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
				return NULL;
			}
			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->usr = (char *) malloc (len+1);
			memset( ret->usr, 0, len+1);

			strncpy( ret->usr, tmp_s, len);
			ret->usr[len] = '\x0';
			
			tmp_s = tmp_s2+1;
			tmp_s2 = strchr( tmp_s,'@');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->pwd = (char *) malloc (len+1);
			memset( ret->pwd, 0, len+1);
			strncpy( ret->pwd, tmp_s, len);
			ret->pwd[len] = '\x0';

			tmp_s = tmp_s2+1;

		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset( ret->addr, 0, len+1 );

			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
			if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
				tmp_s =  tmp_s2;
			} else {
				tmp_s = &url_s[7];
			}
		} else {
			ret->port = DEFAULT_HTTP_PORT;
		}

		if( strchr( tmp_s, '/' )) {

			tmp_s2 = strchr(tmp_s,'/');

			if( !ret->addr ) {
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->addr = (char *) malloc (len+1);
				memset( ret->addr, 0, len+1);
				strncpy( ret->addr, tmp_s, len);
				ret->addr[len] = '\x0';
			}

			tmp_s = tmp_s2;
			ret->path = strdup( tmp_s );
		} else {
			if( !ret->addr )
				ret->addr = strdup( tmp_s );
			ret->path = "/";
		}

	} else {
		free( ret );
		return NULL;
	}

	return ret;
}

int URL_free ( URL *url ) {

	if( !url ) return (0);

	if( url->addr ) free (url->addr);
	if( url->usr ) free (url->usr );
	if( url->pwd ) free (url->pwd );
	if( url->dn ) free (url->dn );
	if( url->path ) free (url->path );

	if( url ) free (url);

	return(1);
}
